from pyramid.config import Configurator
from sqlalchemy import engine_from_config
from pyramid.i18n import TranslationStringFactory


from ringo import DBSession, add_route
from ringo.model import Base
from ringo.lib.i18n import translators

# DO NOT REMOVE THESE LINES
# BEGIN AUTOGENERATED IMPORTS
from plorma.model.task import Task
from plorma.model.comment import Comment
from plorma.model.timelog import Timelog
from plorma.model.workbench import Workbench
# AUTOREPLACEIMPORT
# END AUTOGENERATED IMPORTS

def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.bind = engine
    config = Configurator(settings=settings)
    # Include basic ringo configuration.
    config.include('ringo')
    config.commit()

    # Now configure the application and optionally overwrite previously
    translators.append(TranslationStringFactory('plorma'))
    config.add_translation_dirs('plorma:locale/')
    config.add_static_view('plorma-static', path='plorma:static',
                           cache_max_age=3600)

    # DO NOT REMOVE THESE LINES
    # BEGIN AUTOGENERATED ROUTES
    add_route(config, Task)
    add_route(config, Comment)
    add_route(config, Timelog)
    add_route(config, Workbench)
    # AUTOREPLACEROUTE
    # END AUTOGENERATED ROUTES
    config.add_route('report', 'report')

    # done configurations.
    config.scan()
    return config.make_wsgi_app()
