from datetime import timedelta
import sqlalchemy as sa
from ringo.model import Base
from ringo.model.base import BaseItem, BaseFactory
from ringo.model.modul import ModulItem, _create_default_actions
from ringo.model.mixins import Owned, Meta

STOPPED = 1
RUNNING = 2
PAUSED = 3


class TimelogFactory(BaseFactory):

    def create(self, user=None):
        new_item = BaseFactory.create(self, user)
        # remove group id. Timelogs should only be visible to the user
        new_item.gid = None
        return new_item


class Timelog(BaseItem, Owned, Meta, Base):
    __tablename__ = 'timelogs'
    _modul_id = 11
    id = sa.Column(sa.Integer, primary_key=True)
    seconds = sa.Column(sa.Integer, default=0)
    status = sa.Column(sa.Integer, default=1)
    desc = sa.Column(sa.Text)
    activity = sa.Column(sa.Integer, default=1)

    # ForeignKeys
    task_id = sa.Column(sa.Integer, sa.ForeignKey('tasks.id'))

    # Relations
    task = sa.orm.relationship("Task", backref="timelogs")

    @classmethod
    def get_item_factory(cls):
        return TimelogFactory(cls)

    def __unicode__(self):
        return str(self.id)

    def get_time(self):
        return timedelta(seconds=self.seconds)

    def is_active(self):
        """Returns True if the status of the timelog is either running
        or paused. Else False.

        Available Status:

         * Stopped=1
         * Running=2
         * Paused=3
        :returns: True or False

        """
        return self.status > 1


def init_model(dbsession):
    """Will setup the initial model for the timelog.

    :dbsession: Database session to which the items will be added.
    :returns: None
    """
    modul = ModulItem(name='timelogs')
    modul.clazzpath = "plorma.model.timelog.Timelog"
    modul.label = "Timelog"
    modul.label_plural = "Timelogs"
    modul.display = "hidden"
    modul.actions.extend(_create_default_actions(dbsession))
    dbsession.add(modul)
