import pkg_resources
import os
from datetime import datetime, timedelta
from pyramid.view import view_config
from mako.lookup import TemplateLookup

from ringo.lib.renderer import DTListRenderer
from ringo.model.appointment import Reminders
from ringo.views import handle_history

from ringo.model.base import BaseList
from ringo.lib.helpers import get_week, format_timedelta
from plorma.model.task import Task
from plorma.model.timelog import Timelog


base_dir = pkg_resources.get_distribution("plorma").location
template_dir = os.path.join(base_dir, 'plorma', 'templates')
template_lookup = TemplateLookup(directories=[template_dir],
                                 module_directory='/tmp/plorma_modules')


class QueueRenderer(DTListRenderer):
    def __init__(self, listing):
        DTListRenderer.__init__(self, listing)
        self.template = template_lookup.get_template("internal/taskqueue.mako")
        self.config = self.listing.clazz.get_table_config("taskqueue")


class WorkbenchRenderer(DTListRenderer):
    def __init__(self, listing):
        DTListRenderer.__init__(self, listing)
        self.template = template_lookup.get_template("internal/workbench.mako")
        self.config = self.listing.clazz.get_table_config("workbench")

    def render(self, request):
        """Initialize renderer"""
        now = datetime.now()
        ws, we = get_week(now)
        ds = now.replace(hour=0, minute=0, second=0, microsecond=0)
        de = now.replace(hour=23, minute=59, second=59, microsecond=0)
        values = {'items': self.listing.items,
                  'listing': self.listing,
                  'request': request,
                  '_': request.translate,
                  'tableconfig': self.config,
                  'day_start': ds,
                  'day_end': de,
                  'week_start': ws,
                  'week_end': we,
                  'format_timedelta': format_timedelta
                  }
        return self.template.render(**values)


@view_config(route_name='home', renderer='/index.mako')
def index_view(request):
    handle_history(request)
    values = {}

    if not request.user:
        return values

    # Workbench (only the first and currently only workbench)
    workbench = request.user.workbench[0]
    benchlist = BaseList(Task, request.db, items=workbench.get_tasks())
    values['workbench'] = WorkbenchRenderer(benchlist).render(request)

    # Queue
    queue = BaseList(Task, request.db, request.user)

    # Filter out closed tasks and items which are already on the
    # workbench
    filtered_items = []
    workbench_ids = [t.id for t in workbench.get_tasks()]
    for q in queue.items:
        if (q.get_value("status_id") not in ["Resolved", "Closed", "Verified"]
        and q.id not in workbench_ids):
            filtered_items.append(q)
    queue.items = filtered_items

    # FIXME: Try to use the filter funtion of the baselist.
    # Unfortunately there seems to be no trivial way how to negate a
    # certain word. See commented out code below on how the filtering
    # was intended to be used - and  which does not work (ti)
    # <2013-10-23 13:59>
    ## Filter out closed tasks
    #filter_stack = []
    #filter_stack.append(("[^'Resolved']", "status_id"))
    #filter_stack.append(("[^'Closed']", "status_id"))
    #filter_stack.append(("[^'Verified']", "status_id"))
    ## Filter out items which are already on the Workbench
    #for task in workbench.get_tasks():
    #    re_not(str(task.id))
    #    filter_stack.append(("[^%s]" % task.id, "id"))
    #queue.filter(filter_stack)

    queue.sort("priority_id", "desc", False)
    values['queue'] = QueueRenderer(queue).render(request)

    # Worklog
    times = Timelog.get_item_list(request.db, user=request.user)
    time_today = 0
    time_week = 0
    now = datetime.now()
    week_start, week_end = get_week(now)
    for time in times.items:
        if (time.updated <= week_end
           and time.updated >= week_start):
            time_week += time.seconds
            if time.updated.date() == now.date():
                time_today += time.seconds
    values['time_week'] = format_timedelta(timedelta(seconds=time_week))
    values['time_today'] = format_timedelta(timedelta(seconds=time_today))

    # Reminders
    reminders = Reminders(request.db)
    reminder_renderer = DTListRenderer(reminders)
    values['reminders'] = reminder_renderer.render(request)
    return values
