from datetime import datetime
from pyramid.view import view_config

from formbar.config import Config, load
from formbar.form import Form, Validator

from ringo.views import handle_history

from ringo.model.base import BaseList
from ringo.lib.helpers import get_week, format_timedelta, get_path_to_form_config
from plorma.model.timelog import Timelog

@view_config(route_name='report', renderer='/report/index.mako')
def report_view(request):
    handle_history(request)
    values = {}

    if not request.user:
        return values

    config = Config(load(get_path_to_form_config('report.xml')))
    form_config = config.get_form('report')
    form = Form(form_config, csrf_token=request.session.get_csrf_token())

    all_times = Timelog.get_item_list(request.db, user=request.user)

    times = []

    if request.POST:
        if form.validate(request.params):
            week_start = form.data['start']
            week_end = form.data['end']
        else:
            now = datetime.now()
            week_start, week_end = get_week(now)
            week_start = week_start.date()
            week_end = week_end.date()
    else:
        now = datetime.now()
        week_start, week_end = get_week(now)
        week_start = week_start.date()
        week_end = week_end.date()

    for time in all_times.items:
        if (time.updated.date() <= week_end
            and time.updated.date() >= week_start):
            times.append(time)

    values['times'] = times
    values['form'] = form.render({'start': week_start, 'end': week_end})
    return values
