"""
Overwritten create views. This create view will additionally create a
new workspace per user.
"""
from pyramid.view import view_config
from ringo.views.base import create_
from ringo.views.users import encrypt_password
from ringo.views.json import (
    create_ as json_create,
    )
from ringo.lib.helpers import import_model
from plorma.model.workbench import Workbench
from plorma.model.task import Task

User = import_model('ringo.model.user.User')

def success_callback(request, user):
    _ = request.translate
    wfactory = Workbench.get_item_factory()
    workbench = wfactory.create(user)

    # Now create 4 default Taksk for the user to organise his personal
    # work
    t_factory = Task.get_item_factory()
    t1 = t_factory.create(user)
    t1.title = "%s (%s)" % (_("My personal Tasks"), user.login)
    t1.gid = None
    t1.assigned_id = user.id
    t1.type_id = 0

    t2 = t_factory.create(user)
    t2.title = _("Organisation")
    t2.gid = None
    t2.assigned_id = user.id

    t3 = t_factory.create(user)
    t3.title = _("Miscellaneous")
    t3.gid = None
    t3.assigned_id = user.id

    t4 = t_factory.create(user)
    t4.title = _("Break")
    t4.gid = None
    t4.assigned_id = user.id

    # Put the last tasks as child in the first task
    t1.children.append(t2)
    t1.children.append(t3)
    t1.children.append(t4)

    workbench.add_task(t2)
    workbench.add_task(t3)
    workbench.add_task(t4)

    user.workbench.append(workbench)

    # Create a new workspace for the user
    return encrypt_password(request, user)

@view_config(route_name=User.get_action_routename('create'),
             renderer='/default/create.mako',
             permission='create')
def create(request):
    return create_(User, request, success_callback)

@view_config(route_name=User.get_action_routename('create', prefix="rest"),
             renderer='json',
             request_method="POST",
             permission='create')
def rest_create(request):
    return json_create(User, request, success_callback)
