import logging
import sqlalchemy as sa
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound, HTTPBadRequest

from ringo.views.base import list_, update_, read_
from ringo.views.json import (
    list_ as json_list,
    update_ as json_update,
    read_ as json_read
)
from plorma.model.workbench import Workbench
from plorma.model.task import Task

log = logging.getLogger(__name__)


def _set_task_status(request, task):
    """Will set the status of the task to "Started" if curent state of
    the task is either "Unconfirmed" or "New".

    :request: @todo
    :task: @todo
    :returns: @todo

    """
    print task.status_id, type(task.status_id)
    if task.status_id in ["1","2"]:
        task.status_id = "4"
    return task

def _set_task_assignee(request, task):
    """Will assign the task to the current user if the task has not been
    assigned to someone else before.

    :request: @todo
    :task: @todo
    :returns: @todo

    """
    if task.assigned_id is None:
        task.assigned_id = request.user.id
    return task

def start_task_callback(request, task):
    """Callback which is called after a task has been started.

    :request: @todo
    :task: @todo
    :returns: @todo

    """
    # Get task of the timelog
    task = _set_task_status(request, task)
    task = _set_task_assignee(request, task)
    return task


#                                HTML VIEW                                #
@view_config(route_name=Workbench.get_action_routename('list'),
             renderer='/default/list.mako',
             permission='list')
def list(request):
    return list_(Workbench, request)


@view_config(route_name=Workbench.get_action_routename('update'),
             renderer='/default/update.mako',
             permission='update')
def update(request):
    return update_(Workbench, request)


@view_config(route_name=Workbench.get_action_routename('read'),
             renderer='/default/read.mako',
             permission='read')
def read(request):
    return read_(Workbench, request)


# Custom actions
@view_config(route_name=Workbench.get_action_routename('add_task'),
             permission='update')
def add_task(request):
    # Load the task. Return 400 if the item can not be found.
    factory = Task.get_item_factory()
    id = request.matchdict.get('task')
    try:
        task = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()
    workbench = request.user.workbench[0]
    workbench.add_task(task)

    url = request.route_url("home")
    return HTTPFound(location=url)


@view_config(route_name=Workbench.get_action_routename('del_task'),
             permission='update')
def del_task(request):
    # Load the task. Return 400 if the item can not be found.
    factory = Task.get_item_factory()
    id = request.matchdict.get('task')
    try:
        task = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()
    workbench = request.user.workbench[0]
    workbench.del_task(task)
    url = request.route_url("home")
    return HTTPFound(location=url)


@view_config(route_name=Workbench.get_action_routename('start_task'),
             permission='update')
def start_task(request):
    # Load the task. Return 400 if the item can not be found.
    id = request.matchdict.get('task')
    workbench = request.user.workbench[0]
    # Iterate over all other tasks and pause running tasks
    for task in workbench.tasks:
        if task.id == int(id):
            workbench.start_task(task, request.user)
            start_task_callback(request, task)
        elif task.is_running():
            workbench.pause_task(task, request.user)
    url = request.route_url("home")
    return HTTPFound(location=url)


@view_config(route_name=Workbench.get_action_routename('stop_task'),
             permission='update')
def stop_task(request):
    # Load the task. Return 400 if the item can not be found.
    factory = Task.get_item_factory()
    id = request.matchdict.get('task')
    try:
        task = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()
    workbench = request.user.workbench[0]
    workbench.stop_task(task, request.user)
    url = request.route_url("home")
    return HTTPFound(location=url)


@view_config(route_name=Workbench.get_action_routename('pause_task'),
             permission='update')
def pause_task(request):
    # Load the task. Return 400 if the item can not be found.
    factory = Task.get_item_factory()
    id = request.matchdict.get('task')
    try:
        task = factory.load(id, request.db)
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()
    workbench = request.user.workbench[0]
    workbench.pause_task(task, request.user)
    url = request.route_url("home")
    return HTTPFound(location=url)


#                               REST SERVICE                              #
@view_config(route_name=Workbench.get_action_routename('list', prefix="rest"),
             renderer='json',
             request_method="GET",
             permission='list'
             )
def rest_list(request):
    return json_list(Workbench, request)


@view_config(route_name=Workbench.get_action_routename('read', prefix="rest"),
             renderer='json',
             request_method="GET",
             permission='read')
def rest_read(request):
    return json_read(Workbench, request)


@view_config(route_name=Workbench.get_action_routename('update', prefix="rest"),
             renderer='json',
             request_method="PUT",
             permission='update')
def rest_update(request):
    return json_update(Workbench, request)
